// podpurne funkce pro praci s GC.API
const Chars31Base='0123456789ABCDEFGHJKMNPQRTVWXYZ';

type
  GCAPI_USERINFO=Record
    sUserName:string;         //Nick uzivatele
    sUserCode:string;         //referencni kod uzivatele (PRxxxx)
    sUserID:string;           //ID uzivatele (123445678)
    iUserType:integer;        //typ uzivatele v ciselnem tvaru
    sUserType:string;         //typ uzivatele slovy (Basic, Premium, ...)
    iFind:integer;            //pocet nalezenych kesi
    iHide:integer;            //pocet ukrytych kesi
    iLiteLimit:integer;       //pocet do vycerpani limitu pro stazen kesi v LITE rezimu
    iFullLimit:integer;       //pocet do vycerpani limitu pro stazen kesi v FULL rezimu
  end;

function GcApiUserType2Text(iType:integer):string;
begin
  Result:='Unknown';
  case iType of
  0: Result:='Unknown';
  1: Result:='Basic';
  2: Result:='Charter';
  3: Result:='Premium';
  else ShowMessage('ERR: chybny kod typu uzivatele: '+IntToStr(itype));
  end;
end;

//prevod kodu velikosti kese na text ouzivany v databazi
//https://api.groundspeak.com/documentation#geocache-sizes
function GcApiNumCacheSize2Text(iId:integer):string;
begin
  Result:='??';
  case iId of
  1: Result:='Not chosen';
  2: Result:='Micro';
  8: Result:='Small';
  3: Result:='Regular';  // API ale ma misto Regular Result:='Medium';
  4: Result:='Large';
  5: Result:='Virtual';
  6: Result:='Other';
  else ShowMessage('ERR: chybny kod velikosti kese: '+IntToStr(iId));
  end;
end;


//prevod kodu typu logu na text pouzivany v databazi
//https://api.groundspeak.com/documentation#geocache-types
function GcApiNumLogType2Text(iId:integer):string;
begin
  Result:='??';
  case iId of
  74: Result:='Announcement';
  5: Result:='Archive';
  //: Result:='Archive/Unarchive';
  10: Result:='Attended';
  3: Result:='Didn''t find it';
  23: Result:='Enable Listing';
  2: Result:='Found it';
  7: Result:='Needs Archived';
  45: Result:='Needs Maintenance';
  46: Result:='Owner Maintenance';
  //6: Result:='Permanently Archived';     //zastarale
  68: Result:='Post Reviewer Note';
  //18: Result:='Post Reviewer Note';      //zastarale
  24: Result:='Publish Listing';
  25: Result:='Retract Listing';           //takovy log asi nikdy nedostanu
  //: Result:='Submit for Review';
  22: Result:='Temporarily Disable Listing';
  12: Result:='Unarchive';
  //1: Result:='Unarchive';                //zastarale
  47: Result:='Update Coordinates';
  11: Result:='Webcam Photo Taken';
  9: Result:='Will Attend';
  4: Result:='Write note';
  76: Result:='Submit for Reviewer';       //takovy log asi nikdy nedostanu
  else ShowMessage('ERR: chybny kod typu logu: '+IntToStr(iId));
  end;
end;


//funkce slouzi napr. k prevodu ID logu na jeho kod GLxxxx,
//ID uzivatele na jeho kod PRxxx, ...
function GcApiNum2Code(value:integer): string;
var baseValue: integer;
    i:integer;
begin
  Result:='';
  //stare kody mely kodovani v HEXA
  if(value<=65535) then baseValue:=16
  else begin
    baseValue:=31;
    value:=Value+411120;
  end;
  while (Value>0) do begin
    i:=value mod baseValue;
    value:=value div baseValue;
    Result:=Chars31Base[i+1]+Result;
  end;
end;

function GcApiCode2Num(value:string): integer;
var s:string;
    baseValue,offset,n,i,l:integer;
begin
  Result:=0;
  if(length(value)=0) then exit;
  offset:=0;
  //pokud je potreba, odrizneme prvni 2 znaky idetifikujici typ kodu
  s:=Copy(value,1,2);
  //  cache       user       bookmark   GC log    TB/GC log    Ignore list  watchlist   FP list
  if(s='GC') or (s='PR') or (s='BM') or (s='GL') or (s='TL') or (s='IL') or (s='WL') or (s='FL') then
    value:=copy(value,3,15);
  //stare kody mely kodovani v HEXA
  if((length(value)<=4) and (
    ((value[1]>='0') and (value[1]<='9')) or ((value[1]>='A') and (value[1]<='F')))
    ) then baseValue:=16
  else begin
    baseValue:=31;
    offset:=411120;
  end;
  l:=1;
  for n:=length(value) downto 1 do begin
    i:=pos(value[n],Chars31Base)-1;
    Result:=result+(i*l);
    l:=l*basevalue;
  end;
  Result:=Result-offset;
end;

function GcApiGetUserInfo(sUserName:string):GCAPI_USERINFO;
var s,sJson:string;
    rJson:TSimpleJSon;
    iErr:integer;
begin
  Result.sUserName:=sUserName; Result.sUserCode:=''; Result.sUserID:='';
  Result.iUserType:=-1; Result.sUserType:=''; Result.iFind:=-1; Result.iHide:=-1;
  Result.iLiteLimit:=-1; Result.iFullLimit:=-1;

  s:='v1/users?usernames='+encodeurlelement(AnsiToUtf(sUserName))
    +'&fields=username,referenceCode,findCount,hideCount,favoritePoints,membershipLevelId,'
    //+'avatarUrl,bannerUrl,url,profileText,homeCoordinates,trackableFindCounts,trackableOwnedCounts,'
    +'geocacheFindCounts,geocacheHideCounts,'
    +'geocacheLimits';
  iErr:=GcLiveRest('GET',s,'','',sJson);
  //StringToFile(sJson,GetMyDir()+'\Limits.json');
  if(iErr<>200) then begin
    ShowMessage('ERR: Chyba pri zjistovani informaci o uzivateli');
    exit;
  end;
  sJson:='{root:'+sJson+'}';
  rJson:=TSimpleJson.Create();
  try
    if(rJson.ParseString(sJson)) then begin
      Result.sUserCode:=rJSon.AsString('root[0].referenceCode');
      Result.sUserID:=IntToStr(GcApiCode2Num(Result.sUserCode));
      Result.iUserType:=rJSon.AsInteger('root[0].membershipLevelId');
      Result.sUserType:=GcApiUserType2Text(Result.iUserType);
      Result.iFind:=rJSon.AsInteger('root[0].findCount');
      Result.iHide:=rJSon.AsInteger('root[0].hideCount');
      Result.iLiteLimit:=rJSon.AsInteger('root[0].geocacheLimits.liteCallsRemaining');
      Result.iFullLimit:=rJSon.AsInteger('root[0].geocacheLimits.fullCallsRemaining');
    end;
  finally
    rJson.Free();
  end;
end;
